"""Provides handling of a location."""
from .gateway import Gateway


class Location(object):                                                          # pylint: disable=too-few-public-methods,useless-object-inheritance
    """Provides handling of a location."""

    def __init__(self, client, data=None):
        self.client = client
        self._gateways = []
        self.gateways = {}
        self.locationId = None                                                   # pylint: disable=invalid-name

        if data is not None:
            self.__dict__.update(data['locationInfo'])

            for gw_data in data['gateways']:
                gateway = Gateway(client, self, gw_data)
                self._gateways.append(gateway)
                self.gateways[gateway.gatewayId] = gateway                       # pylint: disable=no-member

    async def status(self):
        """Retrieves the location status."""
        url = ("https://tccna.honeywell.com/WebAPI/emea/api/v1/"
               "location/%s/status?includeTemperatureControlSystems=True" %
               self.locationId)

        async with self.client._session.get(
            url, headers=await self.client._headers()                            # pylint: disable=protected-access
        ) as response:
            response.raise_for_status()
            data = await response.json()

        # Now feed into other elements
        for gw_data in data['gateways']:
            gateway = self.gateways[gw_data['gatewayId']]

            for sys in gw_data["temperatureControlSystems"]:
                system = gateway.control_systems[sys['systemId']]

                system.__dict__.update(
                    {'systemModeStatus': sys['systemModeStatus'],
                     'activeFaults': sys['activeFaults']})

                if 'dhw' in sys:
                    system.hotwater.__dict__.update(sys['dhw'])

                for zone_data in sys["zones"]:
                    zone = system.zones[zone_data['name']]
                    zone.__dict__.update(zone_data)

        return data



"""Provides handling of the hot water zone."""
from .zone import ZoneBase


class HotWater(ZoneBase):
    """Provides handling of the hot water zone."""

    def __init__(self, client, data):
        super(HotWater, self).__init__(client)

        self.dhwId = None                                                        # pylint: disable=invalid-name

        self.__dict__.update(data)

        self.name = ""
        self.zoneId = self.dhwId
        self.zone_type = 'domesticHotWater'

    async def _set_dhw(self, data):
        headers = dict(await self.client._headers())                             # pylint: disable=protected-access
        headers['Content-Type'] = 'application/json'

        url = ("https://tccna.honeywell.com/WebAPI/emea/api/v1"
               "/domesticHotWater/%s/state" % self.dhwId)

        async with self.client._session.put(
            url, json=data, headers=headers
        ) as response:
            response.raise_for_status()

    async def set_dhw_on(self, until=None):
        """Sets the DHW on until a given time, or permanently."""
        if until is None:
            data = {"Mode": "PermanentOverride",
                    "State": "On",
                    "UntilTime": None}
        else:
            data = {"Mode": "TemporaryOverride",
                    "State": "On",
                    "UntilTime": until.strftime('%Y-%m-%dT%H:%M:%SZ')}

        await self._set_dhw(data)

    async def set_dhw_off(self, until=None):
        """Sets the DHW off until a given time, or permanently."""
        if until is None:
            data = {"Mode": "PermanentOverride",
                    "State": "Off",
                    "UntilTime": None}
        else:
            data = {"Mode": "TemporaryOverride",
                    "State": "Off",
                    "UntilTime": until.strftime('%Y-%m-%dT%H:%M:%SZ')}

        await self._set_dhw(data)

    async def set_dhw_auto(self):
        """Sets the DHW to follow the schedule."""
        data = {"Mode": "FollowSchedule",
                "State": "",
                "UntilTime": None}

        await self._set_dhw(data)



"""Test evohomeclient2"""
import requests_mock
from . import EvohomeClient

INSTALLATION_DATA = '''[{
    "locationInfo": {
        "locationId": "locationId"
    },
    "gateways": [
        {
            "gatewayInfo": {
                "location": "location",
                "gatewayId": "gatewayId"
            },
            "temperatureControlSystems": [
                {
                    "systemId": "sysId",
                     "zones": []
                }
            ]
        }
    ]
}]'''
INSTALLATION_DATA_MULTIPLE = '''[{
    "locationInfo": {
        "locationId": "locationId"
    },
    "gateways": [
        {
            "gatewayInfo": {
                "location": "location",
                "gatewayId": "gatewayId"
            },
            "temperatureControlSystems": [
                {
                    "systemId": "sysId",
                    "zones": []
                }
            ]
        },
        {
            "gatewayInfo": {
                "location": "location",
                "gatewayId": "gatewayId"
            },
            "temperatureControlSystems": [
                {
                    "systemId": "sysId",
                    "zones": []
                }
            ]
        }
    ]
}]'''

LOCATION_DATA = '''{
    "locationInfo": {
        "locationId": "locationId"
    },
    "gateways": [
        {
                "gatewayId": "gatewayId",
            "gatewayInfo": {
                "location": "location"
            },
            "temperatureControlSystems": [
                {
                    "systemId": "sysId",
                    "zones": [],
                    "systemModeStatus": "active",
                    "activeFaults": []
                }
            ]
        }
    ]
}'''
LOCATION_DATA_MULTIPLE = '''{
    "locationInfo": {
        "locationId": "locationId"
    },
    "gateways": [
        {
                "gatewayId": "gatewayId",
            "gatewayInfo": {
                "location": "location"
            },
            "temperatureControlSystems": [
                {
                    "systemId": "sysId",
                    "zones": [],
                    "systemModeStatus": "active",
                    "activeFaults": []
                }
            ]
        },
        {
                "gatewayId": "gatewayId",
            "gatewayInfo": {
                "location": "location"
            },
            "temperatureControlSystems": [
                {
                    "systemId": "sysId",
                    "zones": [],
                    "systemModeStatus": "active",
                    "activeFaults": []
                }
            ]
        }
    ]
}'''

AUTH_RESPONSE = '''
  {
    "access_token": "1234",
    "expires_in": 30,
    "refresh_token": "refresh"
  }
'''

USER_RESPONSE = '''
  {
    "name": "name",
    "userId": "userId"
  }
'''

GATEWAY_RESPONSE = '''
{}
'''

@requests_mock.Mocker()
def test_user_account(mock):  # pylint: disable=invalid-name
    """test that user account is successful"""
    mock.post('https://tccna.honeywell.com/Auth/OAuth/Token', status_code=200, text=AUTH_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/userAccount', status_code=200, text=USER_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/installationInfo?userId=userId&includeTemperatureControlSystems=True', status_code=200, text=INSTALLATION_DATA)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/locationId/status?includeTemperatureControlSystems=True', status_code=200, text=LOCATION_DATA)

   # try:
    client = EvohomeClient("username", "password")
    print(client)
    info = client.user_account()
    assert info['name'] == 'name'
    assert info['userId'] == 'userId'

@requests_mock.Mocker()
def test_temperatures(mock):  # pylint: disable=invalid-name
    """test that user account is successful"""
    mock.post('https://tccna.honeywell.com/Auth/OAuth/Token', status_code=200, text=AUTH_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/userAccount', status_code=200, text=USER_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/installationInfo?userId=userId&includeTemperatureControlSystems=True', status_code=200, text=INSTALLATION_DATA)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/locationId/status?includeTemperatureControlSystems=True', status_code=200, text=LOCATION_DATA)

   # try:
    client = EvohomeClient("username", "password")
    print(client)
    list(client.temperatures())

@requests_mock.Mocker()
def test_gateway(mock):  # pylint: disable=invalid-name
    """test that user account is successful"""
    mock.post('https://tccna.honeywell.com/Auth/OAuth/Token', status_code=200, text=AUTH_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/userAccount', status_code=200, text=USER_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/installationInfo?userId=userId&includeTemperatureControlSystems=True', status_code=200, text=INSTALLATION_DATA)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/locationId/status?includeTemperatureControlSystems=True', status_code=200, text=LOCATION_DATA)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/gateway', status_code=200, text=GATEWAY_RESPONSE)

   # try:
    client = EvohomeClient("username", "password")
    client.gateway()

@requests_mock.Mocker()
def test_single_settings(mock):
    """Test can change different statuses"""
    mock.post('https://tccna.honeywell.com/Auth/OAuth/Token', status_code=200, text=AUTH_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/userAccount', status_code=200, text=USER_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/installationInfo?userId=userId&includeTemperatureControlSystems=True', status_code=200, text=INSTALLATION_DATA)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/locationId/status?includeTemperatureControlSystems=True', status_code=200, text=LOCATION_DATA)
    mock.put('https://tccna.honeywell.com/WebAPI/emea/api/v1/temperatureControlSystem/sysId/mode', status_code=200, text='')
    client = EvohomeClient("username", "password", debug=True)

    client.set_status_away()
    client.set_status_eco()
    client.set_status_custom()
    client.set_status_dayoff()
    client.set_status_heatingoff()
    client.set_status_reset()
    client.set_status_normal()

@requests_mock.Mocker()
def test_multi_zone_failure(mock):
    """Confirm that exception is thrown for multiple locations"""
    mock.post('https://tccna.honeywell.com/Auth/OAuth/Token', status_code=200, text=AUTH_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/userAccount', status_code=200, text=USER_RESPONSE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/installationInfo?userId=userId&includeTemperatureControlSystems=True', status_code=200, text=INSTALLATION_DATA_MULTIPLE)
    mock.get('https://tccna.honeywell.com/WebAPI/emea/api/v1/location/locationId/status?includeTemperatureControlSystems=True', status_code=200, text=LOCATION_DATA_MULTIPLE)
    mock.put('https://tccna.honeywell.com/WebAPI/emea/api/v1/temperatureControlSystem/sysId/mode', status_code=200, text='')
    client = EvohomeClient("username", "password", debug=True)

    try:
        client.set_status_away()
        assert False # shouldn't get here
    # pylint: disable=bare-except
    except:
        assert True



"""Provides handling of individual zones"""
import json


class ZoneBase(object):                                                          # pylint: disable=useless-object-inheritance
    """Provides the base for Zones"""

    def __init__(self, client):
        self.client = client
        self.name = None
        self.zoneId = None                                                       # pylint: disable=invalid-name
        self.zone_type = None

    async def schedule(self):
        """Gets the schedule for the given zone"""
        url = ("https://tccna.honeywell.com/WebAPI/emea/api/v1"
               "/%s/%s/schedule" % (self.zone_type, self.zoneId))

        async with self.client._session.get(
            url, headers=await self.client._headers()
        ) as response:
            response.raise_for_status()
            response_data = await response.text()

        mapping = [
            ('dailySchedules', 'DailySchedules'),
            ('dayOfWeek', 'DayOfWeek'),
            ('temperature', 'TargetTemperature'),
            ('timeOfDay', 'TimeOfDay'),
            ('switchpoints', 'Switchpoints'),
            ('dhwState', 'DhwState'),
        ]

        for from_val, to_val in mapping:
            response_data = response_data.replace(from_val, to_val)

        data = json.loads(response_data)
        # change the day name string to a number offset (0 = Monday)
        for day_of_week, schedule in enumerate(data['DailySchedules']):
            schedule['DayOfWeek'] = day_of_week
        return data

    async def set_schedule(self, zone_info):
        """Sets the schedule for this zone"""
        # must only POST json, otherwise server API handler raises exceptions
        try:
            json.loads(zone_info)

        except ValueError as error:
            raise ValueError("zone_info must be valid JSON: ", error)

        headers = dict(await self.client._headers())                             # pylint: disable=protected-access
        headers['Content-Type'] = 'application/json'

        url = ("https://tccna.honeywell.com/WebAPI/emea/api/v1"
               "/%s/%s/schedule" % (self.zone_type, self.zoneId))

        async with self.client._session.put(
            url, data=zone_info, headers=headers
        ) as response:
            response.raise_for_status()

            return await response.json()


class Zone(ZoneBase):
    """Provides the access to an individual zone."""

    def __init__(self, client, data):
        super(Zone, self).__init__(client)

        self.__dict__.update(data)

        self.zone_type = 'temperatureZone'

    async def set_temperature(self, temperature, until=None):
        """Sets the temperature of the given zone"""
        if until is None:
            data = {"SetpointMode": "PermanentOverride",
                    "HeatSetpointValue": temperature,
                    "TimeUntil": None}
        else:
            data = {"SetpointMode": "TemporaryOverride",
                    "HeatSetpointValue": temperature,
                    "TimeUntil": until.strftime('%Y-%m-%dT%H:%M:%SZ')}

        await self._set_heat_setpoint(data)

    async def _set_heat_setpoint(self, data):
        headers = dict(await self.client._headers())                             # pylint: disable=protected-access
        headers['Content-Type'] = 'application/json'

        url = ("https://tccna.honeywell.com/WebAPI/emea/api/v1"
               "/temperatureZone/%s/heatSetpoint" % self.zoneId)

        async with self.client._session.put(
            url, json=data, headers=headers
        ) as response:
            response.raise_for_status()

    async def cancel_temp_override(self):
        """Cancels an override to the zone temperature"""
        data = {"SetpointMode": "FollowSchedule",
                "HeatSetpointValue": 0.0,
                "TimeUntil": None}

        await self._set_heat_setpoint(data)



"""Tests for evohomeclient package"""
from datetime import datetime
import requests_mock
from . import EvohomeClient

UNAUTH_RESPONSE = '''[{
    "code": "Unauthorized",
    "message": "Unauthorized"
}]'''

TASK_ACCEPTED_LIST = '''[{"id": "123"}]'''
TASK_ACCEPTED = '''{"id": "123"}'''

VALID_SESSION_RESPONSE = '''{
  "sessionId": "EE32E3A8-1C09-4A5C-9572-24A088197A38",
  "userInfo": {
    "userID": 123456,
    "username": "username",
    "firstname": "Forename",
    "lastname": "Surname",
    "streetAddress": "Street Address",
    "city": "City",
    "state": "",
    "zipcode": "AB1 2CD",
    "country": "GB",
    "telephone": "",
    "userLanguage": "en-GB",
    "isActivated": true,
    "deviceCount": 0,
    "tenantID": 5,
    "securityQuestion1": "NotUsed",
    "securityQuestion2": "NotUsed",
    "securityQuestion3": "NotUsed",
    "latestEulaAccepted": false
  }
}'''

VALID_ZONE_RESPONSE = '''[
    {
        "locationID": 23456,
        "name": "Home",
        "streetAddress": "Street Address",
        "city": "City",
        "state": "",
        "country": "GB",
        "zipcode": "AB1 2CD",
        "type": "Residential",
        "hasStation": true,
        "devices": [
        {
            "gatewayId": 333444,
            "deviceID": 131313,
            "thermostatModelType": "DOMESTIC_HOT_WATER",
            "deviceType": 96,
            "name": "",
            "scheduleCapable": false,
            "holdUntilCapable": false,
            "thermostat": {
            "units": "Celsius",
            "indoorTemperature": 24.0100,
            "outdoorTemperature": 128.0000,
            "outdoorTemperatureAvailable": false,
            "outdoorHumidity": 128.0000,
            "outdootHumidityAvailable": false,
            "indoorHumidity": 128.0000,
            "indoorTemperatureStatus": "Measured",
            "indoorHumidityStatus": "NotAvailable",
            "outdoorTemperatureStatus": "NotAvailable",
            "outdoorHumidityStatus": "NotAvailable",
            "isCommercial": false,
            "allowedModes": [
                "DHWOn",
                "DHWOff"
            ],
            "deadband": 0.0000,
            "minHeatSetpoint": 5.0000,
            "maxHeatSetpoint": 30.0000,
            "minCoolSetpoint": 50.0000,
            "maxCoolSetpoint": 90.0000,
            "changeableValues": {
                "mode": "DHWOff",
                "status": "Scheduled"
            },
            "scheduleCapable": false,
            "vacationHoldChangeable": false,
            "vacationHoldCancelable": false,
            "scheduleHeatSp": 0.0000,
            "scheduleCoolSp": 0.0000
            },
            "alertSettings": {
            "deviceID": 131313,
            "tempHigherThanActive": false,
            "tempHigherThan": 100.0000,
            "tempHigherThanMinutes": 0,
            "tempLowerThanActive": false,
            "tempLowerThan": -50.0000,
            "tempLowerThanMinutes": 0,
            "faultConditionExistsActive": false,
            "faultConditionExistsHours": 0,
            "normalConditionsActive": true,
            "communicationLostActive": false,
            "communicationLostHours": 0,
            "thermostatAlertActive": false,
            "communicationFailureActive": true,
            "communicationFailureMinutes": 15,
            "deviceLostActive": false,
            "deviceLostHours": 0
            },
            "isUpgrading": false,
            "isAlive": true,
            "thermostatVersion": "03.00.10.06",
            "macID": "001122334455",
            "locationID": 23456,
            "domainID": 28111,
            "instance": 250
        },
        {
            "gatewayId": 333444,
            "deviceID": 121212,
            "thermostatModelType": "EMEA_ZONE",
            "deviceType": 96,
            "name": "RoomName",
            "scheduleCapable": false,
            "holdUntilCapable": false,
            "thermostat": {
            "units": "Celsius",
            "indoorTemperature": 17.5400,
            "outdoorTemperature": 128.0000,
            "outdoorTemperatureAvailable": false,
            "outdoorHumidity": 128.0000,
            "outdootHumidityAvailable": false,
            "indoorHumidity": 128.0000,
            "indoorTemperatureStatus": "Measured",
            "indoorHumidityStatus": "NotAvailable",
            "outdoorTemperatureStatus": "NotAvailable",
            "outdoorHumidityStatus": "NotAvailable",
            "isCommercial": false,
            "allowedModes": [
                "Heat",
                "Off"
            ],
            "deadband": 0.0000,
            "minHeatSetpoint": 5.0000,
            "maxHeatSetpoint": 35.0000,
            "minCoolSetpoint": 50.0000,
            "maxCoolSetpoint": 90.0000,
            "changeableValues": {
                "mode": "Off",
                "heatSetpoint": {
                "value": 15.0,
                "status": "Scheduled"
                },
                "vacationHoldDays": 0
            },
            "scheduleCapable": false,
            "vacationHoldChangeable": false,
            "vacationHoldCancelable": false,
            "scheduleHeatSp": 0.0000,
            "scheduleCoolSp": 0.0000
            },
            "alertSettings": {
            "deviceID": 121212,
            "tempHigherThanActive": true,
            "tempHigherThan": 30.0000,
            "tempHigherThanMinutes": 0,
            "tempLowerThanActive": true,
            "tempLowerThan": 5.0000,
            "tempLowerThanMinutes": 0,
            "faultConditionExistsActive": false,
            "faultConditionExistsHours": 0,
            "normalConditionsActive": true,
            "communicationLostActive": false,
            "communicationLostHours": 0,
            "communicationFailureActive": true,
            "communicationFailureMinutes": 15,
            "deviceLostActive": false,
            "deviceLostHours": 0
            },
            "isUpgrading": false,
            "isAlive": true,
            "thermostatVersion": "03.00.10.06",
            "macID": "001122334455",
            "locationID": 23456,
            "domainID": 28111,
            "instance": 10
        }
        ]
    }
    ]'''
VALID_ZONE_RESPONSE_NO_DHW = '''[
    {
        "locationID": 23456,
        "name": "Home",
        "streetAddress": "Street Address",
        "city": "City",
        "state": "",
        "country": "GB",
        "zipcode": "AB1 2CD",
        "type": "Residential",
        "hasStation": true,
        "devices": [
        {
            "gatewayId": 333444,
            "deviceID": 121212,
            "thermostatModelType": "EMEA_ZONE",
            "deviceType": 96,
            "name": "RoomName",
            "scheduleCapable": false,
            "holdUntilCapable": false,
            "thermostat": {
            "units": "Celsius",
            "indoorTemperature": 17.5400,
            "outdoorTemperature": 128.0000,
            "outdoorTemperatureAvailable": false,
            "outdoorHumidity": 128.0000,
            "outdootHumidityAvailable": false,
            "indoorHumidity": 128.0000,
            "indoorTemperatureStatus": "Measured",
            "indoorHumidityStatus": "NotAvailable",
            "outdoorTemperatureStatus": "NotAvailable",
            "outdoorHumidityStatus": "NotAvailable",
            "isCommercial": false,
            "allowedModes": [
                "Heat",
                "Off"
            ],
            "deadband": 0.0000,
            "minHeatSetpoint": 5.0000,
            "maxHeatSetpoint": 35.0000,
            "minCoolSetpoint": 50.0000,
            "maxCoolSetpoint": 90.0000,
            "changeableValues": {
                "mode": "Off",
                "heatSetpoint": {
                "value": 15.0,
                "status": "Scheduled"
                },
                "vacationHoldDays": 0
            },
            "scheduleCapable": false,
            "vacationHoldChangeable": false,
            "vacationHoldCancelable": false,
            "scheduleHeatSp": 0.0000,
            "scheduleCoolSp": 0.0000
            },
            "alertSettings": {
            "deviceID": 121212,
            "tempHigherThanActive": true,
            "tempHigherThan": 30.0000,
            "tempHigherThanMinutes": 0,
            "tempLowerThanActive": true,
            "tempLowerThan": 5.0000,
            "tempLowerThanMinutes": 0,
            "faultConditionExistsActive": false,
            "faultConditionExistsHours": 0,
            "normalConditionsActive": true,
            "communicationLostActive": false,
            "communicationLostHours": 0,
            "communicationFailureActive": true,
            "communicationFailureMinutes": 15,
            "deviceLostActive": false,
            "deviceLostHours": 0
            },
            "isUpgrading": false,
            "isAlive": true,
            "thermostatVersion": "03.00.10.06",
            "macID": "001122334455",
            "locationID": 23456,
            "domainID": 28111,
            "instance": 10
        }
        ]
    }
    ]'''


@requests_mock.Mocker()
def test_429_returned_raises_exception(mock):  # pylint: disable=invalid-name
    """test that exception is raised for a 429 error"""
    mock.post('http://localhost:5050/WebAPI/api/Session', status_code=429, text='''[
  {
    "code": "TooManyRequests",
    "message": "Request count limitation exceeded, please try again later."
  }
]''')

    try:
        client = EvohomeClient("username", "password",
                               hostname="http://localhost:5050")
        list(client.temperatures())
        # Shouldn't get here
        assert False
    # pylint: disable=bare-except
    except:
        assert True


@requests_mock.Mocker()
def test_valid_login(mock):
    """test valid path"""
    mock.post('http://localhost:5050/WebAPI/api/Session',
              text=VALID_SESSION_RESPONSE)
    mock.get('http://localhost:5050/WebAPI/api/locations?userId=123456&allData=True',
             text=VALID_ZONE_RESPONSE)

    client = EvohomeClient("username", "password",
                           hostname="http://localhost:5050")
    data = list(client.temperatures())

    assert len(data) == 2
    # assert x[1].name == "RoomName"

    assert data == [{'thermostat': 'DOMESTIC_HOT_WATER', 'id': 131313, 'name': '', 'temp': 24.01, 'setpoint': 0, 'status': 'Scheduled', 'mode': 'DHWOff'}, {
        'thermostat': 'EMEA_ZONE', 'id': 121212, 'name': 'RoomName', 'temp': 17.54, 'setpoint': 15.0, 'status': 'Scheduled', 'mode': 'Off'}]


@requests_mock.Mocker()
def test_expired_sessionid(mock):
    """test expired sessionid"""
    mock.post('http://localhost:5050/WebAPI/api/Session',
              text=VALID_SESSION_RESPONSE)
    mock.get('http://localhost:5050/WebAPI/api/locations?userId=123456&allData=True',
             [{'text': UNAUTH_RESPONSE, 'status_code': 401}, {'text': VALID_ZONE_RESPONSE, 'status_code': 200}])

    client = EvohomeClient("username", "password",
                           hostname="http://localhost:5050", user_data={"userInfo": {"userID": "123456"}, "sessionId": "sessionhere"})
    data = list(client.temperatures())

    assert len(data) == 2
    # assert x[1].name == "RoomName"

    assert data == [{'thermostat': 'DOMESTIC_HOT_WATER', 'id': 131313, 'name': '', 'temp': 24.01, 'setpoint': 0, 'status': 'Scheduled', 'mode': 'DHWOff'}, {
        'thermostat': 'EMEA_ZONE', 'id': 121212, 'name': 'RoomName', 'temp': 17.54, 'setpoint': 15.0, 'status': 'Scheduled', 'mode': 'Off'}]


@requests_mock.Mocker()
def test_get_zone_modes(mock):
    """test get zone modes"""
    mock.get('http://localhost:5050/WebAPI/api/locations?userId=123456&allData=True',
             text=VALID_ZONE_RESPONSE)
    client = EvohomeClient("username", "password",
                           hostname="http://localhost:5050", user_data={"userInfo": {"userID": "123456"}, "sessionId": "sessionhere"})
    modes = client.get_modes("RoomName")

    modes2 = client.get_modes(121212)

    assert modes == modes2 == [
        "Heat",
        "Off"
    ]


@requests_mock.Mocker()
def test_set_status(mock):
    """test that statuses can be set correctly"""
    mock.get('http://localhost:5050/WebAPI/api/locations?userId=123456&allData=True',
             text=VALID_ZONE_RESPONSE)
    mock.put('http://localhost:5050/WebAPI/api/evoTouchSystems?locationId=23456',
             text=TASK_ACCEPTED)
    mock.get('http://localhost:5050/WebAPI/api/commTasks?commTaskId=123',
             [{"status_code": 200, "text": """{"state":"pending"}"""}, {"status_code": 200, "text": """{"state":"Succeeded"}"""}])
    client = EvohomeClient("username", "password",
                           hostname="http://localhost:5050", user_data={"userInfo": {"userID": "123456"}, "sessionId": "sessionhere"})
    client.set_status_normal()
    client.set_status_custom()
    client.set_status_eco()
    client.set_status_away()
    client.set_status_dayoff()
    client.set_status_heatingoff(datetime(2019, 10, 10, 10, 10, 10))


@requests_mock.Mocker()
def test_zone_temp(mock):
    """test that zone temps can be set correctly"""
    mock.get('http://localhost:5050/WebAPI/api/locations?userId=123456&allData=True',
             text=VALID_ZONE_RESPONSE)
    mock.put(
        'http://localhost:5050/WebAPI/api/devices/121212/thermostat/changeableValues/heatSetpoint', text=TASK_ACCEPTED)
    mock.put(
        'http://localhost:5050/WebAPI/api/devices/131313/thermostat/changeableValues', text=TASK_ACCEPTED_LIST)
    mock.get('http://localhost:5050/WebAPI/api/commTasks?commTaskId=123',
             [{"status_code": 200, "text": """{"state":"pending"}"""}, {"status_code": 200, "text": """{"state":"Succeeded"}"""}])
    client = EvohomeClient("username", "password",
                           hostname="http://localhost:5050", user_data={"userInfo": {"userID": "123456"}, "sessionId": "sessionhere"})
    client.set_temperature("RoomName", 25)
    client.set_temperature(
        "RoomName", 25, until=datetime(2019, 10, 10, 10, 10, 10))
    client.cancel_temp_override("RoomName")

    mock.get('http://localhost:5050/WebAPI/api/commTasks?commTaskId=123',
             [{"status_code": 200, "text": """{"state":"pending"}"""}, {"status_code": 200, "text": """{"state":"Succeeded"}"""}])
    client.set_dhw_on()
    client.set_dhw_off(datetime(2019, 10, 10, 10, 10, 10))
    client.set_dhw_auto()


@requests_mock.Mocker()
def test_zone_temp_no_dhw(mock):
    """test that zone temps can be set correctly"""
    mock.get('http://localhost:5050/WebAPI/api/locations?userId=123456&allData=True',
             text=VALID_ZONE_RESPONSE_NO_DHW)
    mock.put(
        'http://localhost:5050/WebAPI/api/devices/121212/thermostat/changeableValues/heatSetpoint', text=TASK_ACCEPTED_LIST)
    mock.put(
        'http://localhost:5050/WebAPI/api/devices/131313/thermostat/changeableValues', text=TASK_ACCEPTED_LIST)
    mock.get('http://localhost:5050/WebAPI/api/commTasks?commTaskId=123',
             [{"status_code": 200, "text": """{"state":"pending"}"""}, {"status_code": 200, "text": """{"state":"Succeeded"}"""}])
    client = EvohomeClient("username", "password",
                           hostname="http://localhost:5050", user_data={"userInfo": {"userID": "123456"}, "sessionId": "sessionhere"}, debug=True)
    try:
        client.set_dhw_on()
        assert False
    except:  # pylint: disable=bare-except
        pass



"""Provides handling of a gateway"""
from .controlsystem import ControlSystem


class Gateway(object):                                                           # pylint: disable=too-few-public-methods,useless-object-inheritance
    """Provides handling of a gateway"""

    def __init__(self, client, location, data=None):
        self.client = client
        self.location = location
        self._control_systems = []
        self.control_systems = {}

        if data is not None:
            self.__dict__.update(data['gatewayInfo'])

            for cs_data in data['temperatureControlSystems']:
                control_system = ControlSystem(client, location, self, cs_data)
                self._control_systems.append(control_system)
                self.control_systems[control_system.systemId] = control_system



import os
import sys

from setuptools import setup
from setuptools.command.install import install

VERSION = "0.3.5.post2"

with open("README.md", "r") as fh:
    long_description = fh.read()

class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)

setup(
    name='evohome-async',
    version=VERSION,
    description='An async Python client for connecting to the Evohome webservice',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/zxdavb/evohome-async/',
    download_url='https://github.com/zxdavb/evohome-async/tarball/' + VERSION,
    author='Andrew Stock & David Bonnes',
    author_email='zxdavb@bonnes.me',
    license='Apache 2',
    classifiers=[
        'Development Status :: 4 - Beta',
    ],
    keywords=['evohome', 'total connect comfort', 'round thermostat'],
    packages=['evohomeasync', 'evohomeasync2'],
    install_requires=['aiohttp'],
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)
